#!/usr/bin/perl -w

package acidrip;  
require 5.000;
use strict;
use POSIX; #needed for floor()

use AcidRip::interface;
use AcidRip::signals;
use Data::Dumper;

sub gui_check ($)
{
	if (/SKIN/) {
		my $msg = "You have set gui=yes in your config file. this is not a valid MPlayer option, only there for debugging. Remove it and use gmplayer instead for a gui!";
		message ($msg);
		print $msg . "\n";
	}
}

sub set_warning_text ($$)
{
	my $widget = shift;
	my $warn = shift;
	my $style = new Gtk::RcStyle;
	my $color = $::widgets->{'filesize_spin'}->get_colormap->color_alloc ( {red => ($warn * 65355)} );
	$style->modify_color (['fg', 'text'], 'normal', $color);
	$widget->modify_style ($style);
}

sub hhmmss ($)
{
	my $s = shift;
	my $m = floor $s / 60;
	my $mins = $m % 60;
	my $hour = floor $m / 60;
	my $hhmmss = sprintf ("%d:%02d:%02d", $hour, $mins, $s % 60);
	return $hhmmss;
}

# settings->{blocks}. [0] = desired length of each in seconds else is chapters to fit.
sub split_track
{
	$::settings->{'blocks'} = undef;
	$::settings->{'blocks'} = ();
	my $start = $::settings->{'selected_chapters_start'} || 1;
	my $stop = $::settings->{'selected_chapters_end'} || get_track()->{'chapter'}[-1]->{'ix'};
	my $total_length = get_selection_length();
	
	my $length = 0;
	my $this_block = 1;
	my $old_length = $length;
	$::settings->{'blocks'}[0] = $start - 1;

	foreach my $chapter ($start..$stop)
	{
		my $this_chapter = get_chapter($chapter);
		my $target = $total_length * $this_block / $::settings->{'total_blocks'};
		$length += $this_chapter->{'length'};
		$::settings->{'blocks'}[$this_block] = $chapter;	
		$this_block++ if $old_length < $target && $length > $target; 
		$old_length = $length;
	}

  $::settings->{'UI'} && set_warning_text($::widgets->{'total_blocks_spin'}, (scalar @{$::settings->{'blocks'}}) - 1 < $::settings->{'total_blocks'} ? 1 : 0);
	my $msg = "Blocks: ";
	for (1..@{$::settings->{'blocks'}}-1) {
		$msg .= " $_=[" . ($::settings->{'blocks'}[$_-1]+1) . "-" . ($::settings->{'blocks'}[$_])."] "; 
	}
	message($msg);
	return @{$::settings->{'blocks'}};
}

sub get_track
{
	my $ix = shift || $::settings->{'selected_track'};
	foreach my $t (@{$::dvd->{'track'}}) {
	  return $t if $ix == $t->{'ix'}
	}
	return -1;
}

sub get_chapter
{
	my $ix = shift;
	my $track = get_track(shift);
	foreach my $c (@{$track->{'chapter'}}) {
	  return $c if $ix == $c->{'ix'} 
	}
	return -1;
}

sub message
{
	my $data = shift;

	if ($::settings->{'UI'}) {
		my $bar = $::widgets->{'status_bar'};
		my $context_id=$bar->get_context_id('acidrip');
		$bar->pop($context_id);
		$bar->push($context_id, $data);
		$::widgets->{'mencoder_output_text'}->insert('','','', "AcidRip message - $data\n");
		print LOGFILE "AcidRip message - $data\n" if ($::widgets->{'mencoder_output_saveall_button'}->get_active);
	} else {
		print "AcidRip message - $data\n";
	}
}

sub load_settings_to_interface 
{
	my $settings = shift;
	foreach my $value (keys %{$settings})
	{
		my $entry = join("_", $value, "entry");
		if (defined ($::widgets->{$entry})) {
			$::widgets->{$entry}->set_text($settings->{$value}) if $value ne $::widgets->{$entry}->get_text;
		}
		my $check = join("_", $value, "check");
		if (defined ($::widgets->{$check})) {
			$::widgets->{$check}->set_active($settings->{$value}) if $value ne $::widgets->{$check}->get_active;	
		}
		my $spin = join("_", $value, "spin");
		if (defined ($::widgets->{$spin})) {
			$::widgets->{$spin}->set_value($settings->{$value}) if $value ne $::widgets->{$spin}->get_value_as_int;
		}
	}
}

sub set_setting ($$)
{
	my ($name, $data) = @_;
	my $entry = join("_", $name, "entry");
	my $spin = join("_", $name, "spin");
	$::settings->{$name} = $data;
	$::widgets->{$entry}->set_text($data) if defined $::widgets->{$entry} && $::widgets->{$entry}->editable;
	$::widgets->{$spin}->set_value($data) if defined $::widgets->{$spin} && $::widgets->{$spin}->editable;
}

sub get_parameters 
{
	split_track() if $::settings->{'total_blocks'} > 1;
	set_bitrate() if $::settings->{'this_block'} > 0;

	my %menc = ();
  my $length = get_track()->{'length'};
	
	$menc{'video_options'} = $::settings->{'video_options'};
	$menc{'video'} = "-ovc $::settings->{'video_codec'}";

	$menc{'info'} = "-info srcform=\"DVD ripped by acidrip.sf.net\"";
	$menc{'info'} .= ":name=\"$::settings->{'info_name'}\"" 					if defined $::settings->{'info_name'} && $::settings->{'info_name'} ne '';
	$menc{'info'} .= ":comment=\"$::settings->{'info_comment'}\"" 		if defined $::settings->{'info_comment'} && $::settings->{'info_comment'} ne '';
	$menc{'info'} .= ":artist=\"$::settings->{'info_artist'}\"" 			if defined $::settings->{'info_artist'} && $::settings->{'info_artist'} ne '';
	$menc{'info'} .= ":subject=\"$::settings->{'info_subject'}\"" 		if defined $::settings->{'info_subject'} && $::settings->{'info_subject'} ne '';
	$menc{'info'} .= ":genre=\"$::settings->{'info_genre'}\"" 				if defined $::settings->{'info_genre'} && $::settings->{'info_genre'} ne '';
	$menc{'info'} .= ":copyright=\"$::settings->{'info_copyright'}\"" if defined $::settings->{'info_copyright'} && $::settings->{'info_copyright'} ne '';	
	
	if ($::settings->{'video_codec'} eq 'lavc')
	{
		$menc{'video'} = "-ovc lavc -lavcopts $::settings->{'lavc_options'}:vbitrate=$::settings->{'video_bitrate'}";
		$menc{'video'} .= ":vpass=$::settings->{'video_2pass'}" if ($::settings->{'video_2pass'});
	}
	if ($::settings->{'video_codec'} eq 'divx4')
	{
		$menc{'video'} = "-ovc divx4 -divx4opts $::settings->{'divx4_options'}:br=$::settings->{'video_bitrate'}";
		$menc{'video'} .= ":pass=$::settings->{'video_2pass'}" if ($::settings->{'video_2pass'});
	}
	if ($::settings->{'video_codec'} eq 'xvid') 
	{
		$menc{'video'} = "-ovc xvid -xvidencopts $::settings->{'xvid_options'}:bitrate=$::settings->{'video_bitrate'}";
		$menc{'video'} .= ":pass=$::settings->{'video_2pass'}" if ($::settings->{'video_2pass'}) 
	}

	$menc{'dvd'} = "-dvd $::settings->{'selected_track'}";
	$menc{'dvd'} .= " -dvd-device $::settings->{'dvd_device'}" if $::settings->{'dvd_device'} ne '';
	
	$menc{'dvdp'} = "dvd://$::settings->{'selected_track'}";
	$menc{'dvdp'} .= " -dvd-device $::settings->{'dvd_device'}" if $::settings->{'dvd_device'} ne '';
	
	$menc{'audio_track'} = '';
	$menc{'audio_track'} = "-alang $::settings->{'language'}" if ($::settings->{'selected_audio'} == -1);
	$menc{'audio_track'} = "-nosound" if ($::settings->{'selected_audio'} == -2);
	$menc{'audio_track'} = join(" ", "-aid", $::settings->{'selected_audio'} + 127) if ($::settings->{'selected_audio'} > 0);

	$menc{'audio'} = "-oac $::settings->{'audio_codec'}";
	$menc{'audio'} .= " -lameopts $::settings->{'mp3lame_options'}"	if ($::settings->{'audio_codec'} eq 'mp3lame');

	$menc{'basename'} = $::settings->{'directory'} . $::settings->{'filename'};
  $menc{'basename'} = $menc{'basename'}. "-" . $::settings->{'this_block'} if $::settings->{'this_block'};

	$menc{'output'} = $menc{'basename'} . $::settings->{'filetype'};
	
	$menc{'subp'} = $::settings->{'selected_subp'} > -1 ? "-sid $::settings->{'selected_subp'}" : "";

	$menc{'subout'}= $::settings->{'vobsubout'} ? "-vobsubout $menc{'basename'}" : "";
	
	$menc{'scale'} = $::settings->{'scale_enable'} == 1 ? "scale=$::settings->{'scale_width'}:$::settings->{'scale_height'}" : "";

	$menc{'crop'} = '';
	$menc{'crop'} = "crop=$::settings->{'crop_width'}:$::settings->{'crop_height'}:$::settings->{'crop_horizontal'}:$::settings->{'crop_vertical'}"
		if ($::settings->{'crop_enable'} == 1);

	$menc{'vop_pre'} = '';
	$menc{'vop_pre'} = $::settings->{'vop_pre'} if $::settings->{'vop_pre_enable'} && $::settings->{'vop_pre'} ne '';
	
	$menc{'vop_post'} = '';
	$menc{'vop_post'} = $::settings->{'vop_post'} if $::settings->{'vop_post_enable'} && $::settings->{'vop_post'} ne '';
	
	my $cw = $::settings->{'crop_width'} || get_track()->{'width'};
	my $ch = $::settings->{'crop_height'} || get_track()->{'height'};
	$menc{'rectangle'} = '';
	$menc{'rectangle'} = "-vop crop=$cw:$ch:$::settings->{'crop_horizontal'}:$::settings->{'crop_vertical'}"		
		if ($::settings->{'crop_enable'} == 1);

	$menc{'vop'} = $::settings->{'video_codec'} ne 'copy' ? "-vop $menc{'vop_post'},$menc{'scale'},$menc{'crop'},$menc{'vop_pre'}" : "";
		
	$menc{'embed'} = $::settings->{'embed_preview'} ? " -xy " . $::widgets->{'preview_socket'}->allocation->[2] .
		" -wid " . $::widgets->{'preview_socket'}->window->XWINDOW : '';
	
	$menc{'chapter'} = '';
	if ($::settings->{'total_blocks'} > 1 && $::settings->{'this_block'} > 0) {
			$menc{'chapter'} = "-chapter " . ($::settings->{'blocks'}[$::settings->{'this_block'}-1]+1) . "-". 
				$::settings->{'blocks'}[$::settings->{'this_block'}]; 
		  $length = get_selection_length($::settings->{'blocks'}[$::settings->{'this_block'}-1]+1, 
			  $::settings->{'blocks'}[$::settings->{'this_block'}]);
	}
	elsif ($::settings->{'selected_chapters_start'} > 0 && $::settings->{'selected_chapters_end'} > 0 ) {
		$menc{'chapter'} = "-chapter $::settings->{'selected_chapters_start'}-$::settings->{'selected_chapters_end'}"; 
		$length = get_selection_length($::settings->{'selected_chapters_start'}, $::settings->{'selected_chapters_end'})
	}
	
  my $frames = 10;
  my $sstep = "-sstep " . int $length / (3 * $frames) + 1;
	$sstep = "-ss " . int $length / 3 if $::settings->{'ppc_bug'};
	$menc{'frames'} = "-frames $frames $sstep";
		
	$menc{'cache'} = $::settings->{'cache_directory'} . $::settings->{'filename'} . "-cache";
	$menc{'cache'} = $::settings->{'cache_directory'} . $::settings->{'filename'} . "-" . 
		$::settings->{'this_block'} . "-cache" if $::settings->{'this_block'} != 0;

	$menc{'more_options'} = $::settings->{'more_options'};
	
	return %menc;
}

sub get_command
{
	my $command = shift;
	my %menc = get_parameters();

	return "$::settings->{'mencoder'} $menc{'cache'} $menc{'audio'} $menc{'video'} $menc{'vop'} $menc{'more_options'} $menc{'audio_track'} -o \"$menc{'output'}\""
	if $command eq "mencoder" && $::settings->{'cache'};

	return "$::settings->{'mencoder'} $menc{'dvd'} $menc{'chapter'} $menc{'audio_track'} $menc{'subp'} $menc{'subout'} $menc{'info'} $menc{'audio'} $menc{'video'} $menc{'vop'} -o \"$menc{'output'}\" $menc{'more_options'}"
	if $command eq "mencoder";

	return "$::settings->{'mplayer'} $menc{'rectangle'} -quiet $menc{'dvdp'} $menc{'audio_track'} $menc{'chapter'} $menc{'frames'}	-nocache $menc{'embed'}"
	if $command eq "preview" && $::settings->{'flickbook_preview'} && ! $::settings->{'ppc_bug'};
	
	return "$::settings->{'mplayer'} $menc{'rectangle'} $menc{'dvdp'} $menc{'audio_track'} $menc{'chapter'} $menc{'embed'}"
	if $command eq "preview";

	return "$::settings->{'mplayer'} -quiet \"$menc{'output'}\""
	if $command eq "view";

	return "$::settings->{'mplayer'} $menc{'dvdp'} $menc{'chapter'} -v -v -dumpstream -dumpfile \"$menc{'cache'}\""
	if $command eq "cache";

  return "$::settings->{'mplayer'} -vop cropdetect $menc{'dvdp'} -nosound -vo null $menc{'frames'} -nocache"
	if $command eq "cropdetect";
		
	return "command not found!";
}

sub set_bitrate
{
	return -1 if ! defined $::dvd->{'track'};
	my $length;
	my $size;
	my $bitrate = 0;
	my $audio_rate = 128;
	my $this_track = get_track();
	
	if ( $::settings->{'audio_codec'} eq 'mp3lame' && $::settings->{'mp3lame_options'} =~ /br=(\d+)/ ) {
		$audio_rate = $1
	} elsif ($::settings->{'audio_codec'} eq  'copy') {
		my $channels = $this_track->{'audio'}[$::settings->{'selected_audio'}]->{'channels'};
		$audio_rate = 192 if $channels == 2;
		$audio_rate = 384 if $channels == 6;
	} else {
		message("no audio bitrate found, 128 assumed")
	}
  
	if ($::settings->{'this_block'}) {
	  $length = get_selection_length($::settings->{'blocks'}[$::settings->{'this_block'}-1]+1, $::settings->{'blocks'}[$::settings->{'this_block'}]);
    $size = $::settings->{'filesize'};
	} elsif ($::settings->{'total_blocks'} > 1) {
	  $length = get_selection_length();
		$size = $::settings->{'filesize'} * (scalar @{$::settings->{'blocks'}}-1);
	} else {
	  $length = get_selection_length();
		$size = $::settings->{'filesize'};
	}
		
  if ($size == 0 || $length == 0 || $size!~/^\d+$/) { $bitrate = -1 } 
	else { $bitrate = int((($size*8192)/$length) - $audio_rate) }
	$bitrate = 16000 if $bitrate > 16000;
	if ($bitrate < 0 ) { message("Error setting bitrate") }
	set_setting('video_bitrate', $bitrate);
	set_bpp();
}

sub set_bpp
{
	return -1 if ! defined $::dvd->{'track'};
	
	my $bpp = 0;
	my $fps = get_track()->{'fps'};
	my $wo = get_track()->{'width'};
	my $ho = get_track()->{'height'};
	my $ws = $::settings->{'scale_width'};
	my $hs = $::settings->{'scale_height'};
	my $br = $::settings->{'video_bitrate'};
	my $wc = $::settings->{'crop_width'}  || $wo;
	my $hc = $::settings->{'crop_height'} || $ho;
	my $aa = eval(get_track()->{'aspect'}); #eval to allow fraction string to equate to a float
	
	if (defined $aa && $br * $hc * $wc * $wo * $ho * $fps) # moronic test for potential divide by zero!
	{
		$hs = $hs || $hc;
		$ws = $ws || $ho * $wc * $aa / $wo;
		$hs = $hc if $hs == -1;
		$ws = $wc if $ws == -1;
		$hs = ($hc * $wo * $ws) / ($wc * $ho * $aa) if $hs == -2;
		$ws = ($wc * $ho * $hs * $aa) / ($hc * $wo) if $ws == -2;

		$ws = sprintf("%i", $ws);
		$hs = sprintf("%i", $hs);
		$bpp = sprintf("%.3f", ($br * 1000) / ($hs * $ws * $fps)) if $ws * $hs;

		if ( defined $::widgets && $ws > 0 && $hs > 0 ) {
			$::widgets->{'scale_height_estimate_entry'}->set_text($hs);
			$::widgets->{'video_bpp_entry'}->set_text($bpp);
		} #else { message ("Estimated video size $ws x $hs, bits per pixel: $bpp") }
	}
	return $ws, $hs, $wc, $hc, $wo, $ho;
}

sub get_available_codecs
{
	my $settings = shift;
	my $menc = $settings->{'mencoder'};
	if ( open (MENC_VIDEO, "$menc -ovc help 2>&1 |")) {
		while (<MENC_VIDEO>) {$settings->{'available_video'}{$1}="$1 - $2" if $_=~/\s+(\w+)\s+-\s+(.+)/}
		if (defined $::widgets) { 
			$::widgets->{'video_codec_combo'}->set_popdown_strings(keys %{$settings->{'available_video'}})
		}
		close MENC_VIDEO
	} else { message("Mencoder codec test failed. Is MPlayer installed properly?") }
	
	if (open (MENC_AUDIO, "$menc -oac help 2>&1 |")) {
		my $got_mp3 = 0;
		while (<MENC_AUDIO>) {
			if ( $_=~/\s+(\w+)\s+-(.*)/ ) {
				$settings->{'available_audio'}{$1}="$1 - $2";
				$got_mp3 = 1 if $1 eq "mp3lame"
			}
		}
		if (defined $::widgets) { 
			$::widgets->{'audio_codec_combo'}->set_popdown_strings(keys %{$settings->{'available_audio'}})
		}
		print "AcidRip could NOT find MP3 support in MPlayer! if you do want MP3 sound in your films," . 
			"you will need to recompile MPlayer accordingly. See the docs on the MPlayer site.\n\n" if ! $got_mp3;
		close MENC_AUDIO
	} else { message("Mencoder codec test failed. Is MPlayer installed properly?") }
}

sub get_selection_length
{
	my ($start, $end) = @_;
	$start = ( $::settings->{'selected_chapters_start'} || 1 ) if ! $start;
	$end = ( $::settings->{'selected_chapters_end'} || get_track->{'chapter'}[-1]->{'ix'} ) if ! $end;
	my $length = 0;
	for my $chapter ($start..$end) {
		$length += get_chapter($chapter)->{'length'};
	}
	return $length;
}

sub read_disc
{
	use vars qw(%lsdvd);
	system("$::settings->{'lsdvd'} -h > /dev/null 2> /dev/null");
	if (! $?) {
		eval (`$::settings->{'lsdvd'} -xp $::settings->{'dvd_device'} 2>/dev/null`);
		$::dvd = {%lsdvd};
		$Data::Dumper::Indent = 0;
		message(Dumper {%lsdvd});
		$::dvd->{'status'} = "DVD read ok";
		$Data::Dumper::Indent = 2;
	}
	$::dvd->{'status'} = "lsdvd not found" if $? >> 8 == 127;
	$::dvd->{'status'} = "DVD device not found" if $? >> 8 == 1;
	$::dvd->{'status'} = "DVD not found. Drive empty?" if $? >> 8 == 2;
	$::dvd->{'status'} = "Can't read DVD. Not a valid DVD drive!" if $? >> 8 == 3;
	$::dvd->{'status'} = "Can't read DVD track. Faulty Disc?" if $? >> 8 == 4;
	return $?;
}

sub find_crop ($) {
	open STDIN,  '/dev/null' or message("Can't read /dev/null: $!. May fail if in background");

	my $crop_output;
	if (defined $::dvd->{'track'})
	{
    my %crop;
		message("Running " . get_command("cropdetect"));
	  open (CROP, get_command("cropdetect") . " 2>&1 |");
    while (<CROP>) { 
			if ($_ =~ /crop area.*\(-vop (crop=\d*:\d*:\d*:\d*)/) { $crop{$1}++ }
			$crop_output .= $_;
			gui_check($_);
		}
	  my @order = sort { $crop{$b} <=> $crop{$a} } keys %crop;
		close CROP;
    return ( $order[0], $crop_output )
  } else { return ( -1, "crop detect failed" ) }
}

sub cache
{
	#fork proofing
	open STDIN,  '/dev/null' or message("Can't read /dev/null: $!. May fail if in background");

	my $cache = shift;
	$cache=~/"(.+)"/;
	my $file = $1;
	my $this_cell = -1;
	my $size = 0;
	my $start_chapter;
	my $stop_chapter;
	my $this_track = get_track();

	if ($::settings->{'total_blocks'} > 1) {
		$start_chapter = $::settings->{'blocks'}[$::settings->{'this_block'}];
		$stop_chapter = $::settings->{'blocks'}[$::settings->{'this_block'} + 1];
	} else {
		$start_chapter = $::settings->{'selected_chapters_start'};
		$stop_chapter = $::settings->{'selected_chapters_end'};
		$start_chapter = 1 if $start_chapter == 0;
	  $stop_chapter = $this_track->{'chapter'}[-1]->{'ix'} if $stop_chapter == 0;
	}
	my $total_chapters = $stop_chapter - $start_chapter + 1;	
	
	$::settings->{'mencoder_pid'} = open (CACHE, "nice $cache 2>&1 |");
  while (<CACHE>) {
		if (/### CELL (\d+)/) {
			if ($this_cell != $1 + 1) {			
				$this_cell = $1 + 1;
				my $this_chapter = get_track()->{'cell'}[$this_cell]->{'chapter'};
				$::widgets->{'cache_chapter'}->set_text(sprintf("%d (%d/%d)", $this_chapter, $this_chapter + 1 - $start_chapter, $total_chapters));
			}
			$::widgets->{'cache_size'}->set_text( sprintf("%1.2f", ( -s $file) / 1048576) . "mb") if -e $file;
		}
		while (Gtk->events_pending()) {Gtk->main_iteration()};
	}
	close CACHE;
	return $file;
}

sub encode
{
	#fork proofing
	open STDIN,  '/dev/null' or message("Can't read /dev/null: $!. May fail if in background");
	
	my $menc = shift;

	if (defined $::widgets) {
		
		if ($::settings->{'mencoder_pid'} = open(MENCODER, "$menc 2>&1 |")) {
			$/="\r";
			while (<MENCODER>)
			{
				if (/^Pos:\s*(\d+.\ds)\s+(\d+)f\s+\(\s*(\d+)%\)\s+(\d+fps)\sTrem:\s+(\d+min)\s+(\d+mb).+\[([\d:]+)\]/) {
					$::widgets->{'menc_seconds'}->set_text($1)	 if ($::widgets->{'menc_seconds'}->get ne $1);
					$::widgets->{'menc_fps'}->set_text($4)			 if ($::widgets->{'menc_fps'}->get ne $4);
					$::widgets->{'menc_filesize'}->set_text($6)  if ($::widgets->{'menc_filesize'}->get ne $6);
					$::widgets->{'menc_time'}->set_text($5)		   if ($::widgets->{'menc_time'}->get ne $5);
					$::widgets->{'menc_bitrate'}->set_text($7)	 if ($::widgets->{'menc_bitrate'}->get ne $7);
					$::widgets->{'menc_progress'}->set_percentage($3/100);
				} else { s/\r/\n/g; $::widgets->{'mencoder_output_text'}->insert('','','', $_)}
				while (Gtk->events_pending()) {Gtk->main_iteration()};
			}
			$/="\n";
		} else { message("Mencoder failed, is it properly installed?") }
	} else { 
		message( "Mencoder failed, is it properly installed?") if ! ( $::settings->{'mencoder_pid'} = open(MENCODER, "$menc"));
	}
	close MENCODER;
}

package acidrip_settings;

use vars qw($defaults);

$defaults = {
	'dvd_device'        => '/dev/dvd',
	'selected_track'    =>  0,
	'selected_chapters_start' => 0,
	'selected_chapters_end'   => 0,
	'filesize'          =>  700,
	'total_blocks'      =>  1,
	'this_block'        =>  0,
	'blocks'            => [],
	'selected_audio'    => -1,
	'selected_subp'	    => -1,
	'audio_codec'       => 'mp3lame',
	'audio_options'     => '',
	'mp3lame_options'   => 'abr:br=128',
	'video_codec'       => 'lavc',
	'lavc_options'      => 'vcodec=mpeg4:vhq:vqmin=2',
	'divx4_options'	    => '',
	'xvid_options'	    => '',
	'video_options'	    => '',
	'video_bitrate'	    =>  0,
	'directory'         => $ENV{HOME},
	'filename'          => 'unknown',
	'filetype'          => '.avi',
	'scale_enable'      =>  1,
	'scale_auto'        =>  1,
	'scale_height'      => -2,
	'scale_width'       =>  480,
	'crop_enable'       =>  1,
	'crop_height'       =>  0,
	'crop_width'        =>  0,
	'crop_vertical'     =>  0,
	'crop_horizontal'   =>  0,
	'vop_pre'           =>  'pp=de',
	'vop_pre_enable'    =>  1,
	'vop_post'          =>  '',
	'vop_post_enable'		=>  0,
	'video_2pass'       =>	0,
	'more_options'      => '',
	'language'          => 'en',
	'mencoder'          => 'mencoder',
	'mplayer'           => 'mplayer',
	'lsdvd'             => 'lsdvd',
	'available_audio'   => {},
	'available_video'   => {},
	'mencoder_pid'	    =>  0,
	'autoload'          =>  0,
	'overwrite'         =>  0,
	'enforce_space'			=>  1,
	'cache'             =>  0,
	'cache_directory'   =>  '/tmp/',
	'del_cache'         =>  0,
	'tooltips'          =>  1,
	'busy'              =>  0,
	'UI'                =>  1,
	'compact'						=>  1,
	'enough_space'			=>  1,
	'ppc_bug'           =>  0,
	'embed_preview'     =>  1,
	'vobsubout'         =>  0
};
	
sub new
{
	my $class = shift;
	my $settings = {};
	foreach my $key (keys %{$defaults}) 
		{$settings->{$key}=$defaults->{$key}}
	bless ($settings, $class);
	return $settings;
}

sub get_default ($$)
{
	my ($settings, $value) = @_;
	return $defaults->{$value} if (defined $settings->{$value});
	return "unknown setting! - $value"
}

sub restore_settings
{
	my $settings = shift;
	foreach my $key (keys %{$defaults}) 
		{$settings->{$key}=$defaults->{$key} unless $key eq "selected_track"}
	return $settings;
}

sub save_settings
{
	my $settings = shift;
	my @save = ('dvd_device', 'filesize', 'directory', 'audio_codec', 'mp3lame_options', 'video_codec', 'scale_enable', 'autoload', 'filetype',
	            'scale_auto', 'scale_height', 'scale_width', 'crop_enable', 'language', 'mencoder', 'mplayer', 'lsdvd', 'tooltips', 'video_2pass',
							'lavc_options', 'xvid_options', 'divx4_options', 'more_options', 'total_blocks', 'overwrite', 'cache', 'cache_directory',
						  'vop_filters_enable', 'vop_pre', 'vop_post', 'del_cache', 'ppc_bug', 'compact', 'flickbook_preview', 'enforce_space', 'vobsubout');
	if (open (CONFIGFILE, '>', "$ENV{HOME}/.acidriprc"))
	{
		foreach my $key (@save) {if ($settings->{$key} ne '') {
			print CONFIGFILE "$key = $settings->{$key}\n"}
		}
		close CONFIGFILE;
	} else {
		acidrip::message("Can't open config file for writing!\n")
	}
}

sub load_settings
{
	my $settings = shift;
	if (open (CONFIGFILE, "$ENV{HOME}/.acidriprc")) {
		while(<CONFIGFILE>) {
			if ($_=~/(\w+)\s=\s(.+)/) { $settings->{$1}=$2 }
		}
		close CONFIGFILE;
	} else { 
		acidrip::message("No configuration file found, nevermind.")
	}
	return $settings;
}

1;
